@@ -289,7 +289,6 @@ print.summary.STA <- function(x,
 #'
 #' @family STA
 #'
-#' @import ggplot2
 #' @importFrom grDevices topo.colors colorRampPalette
 #' @export
 plot.STA <- function(x,
@@ -383,28 +382,40 @@ plot.STA <- function(x,
         plots$p1 <- ggplot(data = plotDat,
                            aes_string(x = "residuals",
                                       y = "(..count..)/sum(..count..)")) +
-          geom_histogram(fill = "cyan", col = "black", bins = 10,
+          geom_histogram(fill = "darkgrey", color = "grey50", bins = 10,
                          boundary = 0) +
-          scale_y_continuous(labels = function(x) {paste0(100 * x, "%")}) +
-          labs(y = "Frequency", x = "Residuals")
+          scale_y_continuous(expand = c(0, 0, 0, 0.05),
+                             labels = function(x) {paste0(100 * x, "%")}) +
+          labs(y = "Frequency", x = "Residuals") +
+          theme(panel.background = element_blank(),
+                panel.grid = element_blank(),
+                panel.border = element_rect(color = "black", fill = NA))
         ## Plot QQ plot of residuals.
-        plots$p2 <- ggplot(data = plotDat,
-                           aes_string(sample = "residuals")) +
-          stat_qq(col = "blue") +
-          labs(y = "Residuals", x = "Normal quantiles")
+        plots$p2 <- ggplot(data = plotDat, aes_string(sample = "residuals")) +
+          stat_qq(shape = 1, alpha = 0.7) +
+          labs(y = "Residuals", x = "Normal quantiles") +
+          theme(panel.background = element_blank(),
+                panel.grid = element_blank(),
+                panel.border = element_rect(color = "black", fill = NA))
         ## Plot residuals vs fitted values.
         plots$p3 <- ggplot(data = plotDat,
                            aes_string(x = "fitted", y = "residuals")) +
-          geom_point(col = "blue", shape = 1) +
-          geom_smooth(method = "loess", col = "red") +
+          geom_point(shape = 1, alpha = 0.7) +
+          geom_smooth(method = "loess", color = "red") +
           geom_hline(yintercept = 0) +
-          labs(y = "Residuals", x = "Fitted values")
+          labs(y = "Residuals", x = "Fitted values") +
+          theme(panel.background = element_blank(),
+                panel.grid = element_blank(),
+                panel.border = element_rect(color = "black", fill = NA))
         ## Plot absolute value of residuals vs fitted values.
         plots$p4 <- ggplot(data = plotDat,
                            aes_string(x = "fitted", y = "abs(residuals)")) +
-          geom_point(col = "blue", shape = 1) +
-          geom_smooth(method = "loess", col = "red") +
-          labs(y = "|Residuals|", x = "Fitted values")
+          geom_point(shape = 1, alpha = 0.7) +
+          geom_smooth(method = "loess", color = "red") +
+          labs(y = "|Residuals|", x = "Fitted values") +
+          theme(panel.background = element_blank(),
+                panel.grid = element_blank(),
+                panel.border = element_rect(color = "black", fill = NA))
         if (output) {
           ## do.call is needed since grid.arrange doesn't accept lists as input.
           do.call(gridExtra::grid.arrange,
@@ -447,11 +458,14 @@ plot.STA <- function(x,
             spatTr$fit
           ## Melt to get the data in ggplot shape. Rows and columns in the
           ## spatial trend coming from SpATS are swapped so therefore use t()
-          plotDatSpat <- reshape2::melt(t(spatTrDat),
-                                        varnames = c("colCoord", "rowCoord"))
+          spatTrDat <- as.data.frame(t(spatTrDat))
+          plotDatSpat <- reshape(spatTrDat, direction = "long",
+                                 varying = list(rowCoord = colnames(spatTrDat)),
+                                 timevar = "rowCoord", idvar = "colCoord",
+                                 v.names = "value")
           ## Add true values for columns and rows for plotting.
-          plotDatSpat$colCoord <- spatTr$col.p
-          plotDatSpat$rowCoord <- rep(x = spatTr$row.p, each = p1 * nCol)
+          plotDatSpat[["colCoord"]] <- spatTr$col.p
+          plotDatSpat[["rowCoord"]] <- rep(x = spatTr$row.p, each = p1 * nCol)
           ## Remove missings from data.
           plotDatSpat <- remove_missing(plotDatSpat, na.rm = TRUE)
         }
@@ -502,7 +516,7 @@ plot.STA <- function(x,
                               title = legends[5], colors = colors)
         plots$p6 <- ggplot(data = plotDat) +
           geom_histogram(aes(x = residuals),
-                         fill = "white", col = "black", bins = 10,
+                         fill = "white", color = "black", bins = 10,
                          boundary = 0, na.rm = TRUE) +
           ## Remove empty space between ticks and actual plot.
           scale_x_continuous(expand = c(0, 0)) +
@@ -527,6 +541,9 @@ plot.STA <- function(x,
   invisible(p)
 }
 
+# darkgrey ----------------------------------------------------------------
+
+
 #' Helper function for creating field plots.
 #'
 #' @noRd
@@ -629,6 +646,10 @@ report.STA <- function(x,
   if (nchar(Sys.which("pdflatex")) == 0) {
     stop("An installation of LaTeX is required to create a pdf report.\n")
   }
+  if (!is.null(outfile) && (!is.character(outfile) || length(outfile) > 1 ||
+                            tools::file_ext(outfile) != "pdf")) {
+    stop("Invalid output filename provided.\n")
+  }
   trials <- chkTrials(trials, x)
   chkChar(traits)
   ## Generate a single timestamp for all files.
@@ -656,7 +677,7 @@ report.STA <- function(x,
         ## report name has to be adapted.
         if (!is.null(outfile)) {
           ## Add trial and trait info before file extension.
-          outExt <- tools::file_ext("test.pdf")
+          outExt <- tools::file_ext(outfile)
           outLen <- nchar(outfile)
           outfileTr <- paste0(substring(text = outfile, first = 1,
                                         last = outLen - nchar(outExt) - 1),
@@ -666,8 +687,8 @@ report.STA <- function(x,
                               "_", timeStamp, ".pdf")
         }
         createReport(x = modTr, reportName = "modelReport.Rnw",
-                     outfile = outfileTr, ..., trial = trial, trait = trait,
-                     descending = descending)
+                     reportPackage = "statgenSTA", outfile = outfileTr, ...,
+                     trial = trial, trait = trait, descending = descending)
       } # End loop over traits.
     } # End loop over what.
   } # End loop over trials.
